﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Runtime.InteropServices;
using System.Drawing;
using System.Windows.Forms;
using ControlLibraryExt;

namespace ActiveXLibrary
{
    [Guid("78E8D592-ABE1-400E-AEE5-84800A0715D5")]
    public class ActiveXAlert : IObjectSafety
    {
        #region IObjectSafety 成员

        private const string _IID_IDispatch = "{00020400-0000-0000-C000-000000000046}";
        private const string _IID_IDispatchEx = "{a6ef9860-c720-11d0-9337-00a0c90dcaa9}";
        private const string _IID_IPersistStorage = "{0000010A-0000-0000-C000-000000000046}";
        private const string _IID_IPersistStream = "{00000109-0000-0000-C000-000000000046}";
        private const string _IID_IPersistPropertyBag = "{37D84F60-42CB-11CE-8135-00AA004BB851}";

        private const int INTERFACESAFE_FOR_UNTRUSTED_CALLER = 0x00000001;
        private const int INTERFACESAFE_FOR_UNTRUSTED_DATA = 0x00000002;
        private const int S_OK = 0;
        private const int E_FAIL = unchecked((int)0x80004005);
        private const int E_NOINTERFACE = unchecked((int)0x80004002);

        private bool _fSafeForScripting = true;
        private bool _fSafeForInitializing = true;


        public int GetInterfaceSafetyOptions(ref Guid riid, ref int pdwSupportedOptions, ref int pdwEnabledOptions)
        {
            int Rslt = E_FAIL;

            string strGUID = riid.ToString("B");
            pdwSupportedOptions = INTERFACESAFE_FOR_UNTRUSTED_CALLER | INTERFACESAFE_FOR_UNTRUSTED_DATA;
            switch (strGUID)
            {
                case _IID_IDispatch:
                case _IID_IDispatchEx:
                    Rslt = S_OK;
                    pdwEnabledOptions = 0;
                    if (_fSafeForScripting == true)
                        pdwEnabledOptions = INTERFACESAFE_FOR_UNTRUSTED_CALLER;
                    break;
                case _IID_IPersistStorage:
                case _IID_IPersistStream:
                case _IID_IPersistPropertyBag:
                    Rslt = S_OK;
                    pdwEnabledOptions = 0;
                    if (_fSafeForInitializing == true)
                        pdwEnabledOptions = INTERFACESAFE_FOR_UNTRUSTED_DATA;
                    break;
                default:
                    Rslt = E_NOINTERFACE;
                    break;
            }

            return Rslt;
        }

        public int SetInterfaceSafetyOptions(ref Guid riid, int dwOptionSetMask, int dwEnabledOptions)
        {
            int Rslt = E_FAIL;

            string strGUID = riid.ToString("B");
            switch (strGUID)
            {
                case _IID_IDispatch:
                case _IID_IDispatchEx:
                    if (((dwEnabledOptions & dwOptionSetMask) == INTERFACESAFE_FOR_UNTRUSTED_CALLER) &&
                            (_fSafeForScripting == true))
                        Rslt = S_OK;
                    break;
                case _IID_IPersistStorage:
                case _IID_IPersistStream:
                case _IID_IPersistPropertyBag:
                    if (((dwEnabledOptions & dwOptionSetMask) == INTERFACESAFE_FOR_UNTRUSTED_DATA) &&
                            (_fSafeForInitializing == true))
                        Rslt = S_OK;
                    break;
                default:
                    Rslt = E_NOINTERFACE;
                    break;
            }

            return Rslt;
        }

        #endregion

        #region JS调用方法

        /// <summary>
        /// 显示提示信息
        /// </summary>
        /// <param name="title">标题</param>
        /// <param name="text">内容</param>
        /// <param name="messsageType">信息类型</param>
        /// <param name="customAlertBackColor">自定义信息背景html颜色</param>
        /// <param name="customAlertTextColor">自定义信息文本html颜色</param>
        public void ShowAlert(string title, string text, string messsageType, string customAlertBackColor, string customAlertTextColor)
        {
            try
            {
                if (!DesktopAlert.Notify.Visible)
                    DesktopAlert.ShowNotify();

                Color? backColor = null;
                if (!String.IsNullOrWhiteSpace(customAlertBackColor))
                    backColor = ColorTranslator.FromHtml(customAlertBackColor.Trim());

                Color? textColor = null;
                if (!String.IsNullOrWhiteSpace(customAlertTextColor))
                    textColor = ColorTranslator.FromHtml(customAlertTextColor.Trim());

                DesktopAlert.MessageType type = (DesktopAlert.MessageType)Enum.Parse(typeof(DesktopAlert.MessageType), messsageType);
                DesktopAlert.ShowAlert(title, text, type, backColor, textColor);
            }
            catch
            {

            }
        }

        /// <summary>
        /// 关闭通知栏
        /// </summary>
        public void CloseNotify()
        {
            try
            {
                DesktopAlert.HideNotify();
            }
            catch
            {

            }
        }

        #endregion

    }
}
